import 'dart:convert';
import 'dart:io';

import 'package:dio/dio.dart';
import 'package:flutter/foundation.dart';

import '../../core/core.dart';
import '../../m4e_forms/m4e_forms.dart';
import '../../models/models.dart';

abstract class GroupApi {
  Future<M4eUniqueId> createGroup(M4eGroupForm m4eGroupForm);

  Future<List<M4eUniqueId>> addGroupEntityRoles(
      M4eGroupEntityRoleForm groupEntityRoleForm);
}

class GroupApiImpl implements GroupApi {
  final Dio _client;

  GroupApiImpl({
    @required Dio client,
    @required M4eLoggingInterceptors loggingInterceptor,
    @required M4eHeaderInterceptors headerInterceptor,
  })  : assert(client != null),
        assert(loggingInterceptor != null),
        assert(headerInterceptor != null),
        _client = client {
    _client?.interceptors?.add(loggingInterceptor);
    _client?.interceptors?.add(headerInterceptor);
  }

  @override
  Future<M4eUniqueId> createGroup(M4eGroupForm m4eGroupForm) async {
    try {
      final _response = await _client.put(
        '/groups/${m4eGroupForm.id.toString()}',
        data: json.encode(m4eGroupForm.toJson()),
      );

      return M4eUniqueId.fromJson(_response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.CONNECT_TIMEOUT) {
        throw M4eExceptionMessages.kConnectionTimeOutException;
      }

      if (e.response?.statusCode == 403) {
        throw M4eExceptionMessages.kUnauthorized;
      }

      if (e.response?.statusCode == 503) {
        throw M4eExceptionMessages.kServiceUnavailableException;
      }

      if (e.response?.statusCode == 500) {
        throw M4eExceptionMessages.kUndefinedException;
      }

      if (e.type == DioErrorType.RESPONSE) {
        throw M4eServerException.fromJson(e.response.data);
      }

      throw M4eServerException.fromString(e.message.toString());
    } on SocketException {
      throw M4eExceptionMessages.kNoInternetConnectionException;
    } on HttpException {
      throw M4eExceptionMessages.kHttpException;
    } on FormatException {
      throw M4eExceptionMessages.kHttpFormatException;
    } catch (e) {
      throw M4eServerException.fromString(e.toString());
    }
  }

  @override
  Future<List<M4eUniqueId>> addGroupEntityRoles(
      M4eGroupEntityRoleForm groupEntityRoleForm) async {
    try {
      final _response = await _client.put(
        '/groups/${groupEntityRoleForm.groupId.toString()}/user-roles',
        data: json.encode(groupEntityRoleForm.toJson()),
      );

      final _ids = _response.data['ids'] as List<String>;

      return _ids?.map((e) => M4eUniqueId.fromUniqueString(e))?.toList() ?? [];
    } on DioError catch (e) {
      if (e.type == DioErrorType.CONNECT_TIMEOUT) {
        throw M4eExceptionMessages.kConnectionTimeOutException;
      }

      if (e.response?.statusCode == 403) {
        throw M4eExceptionMessages.kUnauthorized;
      }

      if (e.response?.statusCode == 503) {
        throw M4eExceptionMessages.kServiceUnavailableException;
      }

      if (e.response?.statusCode == 500) {
        throw M4eExceptionMessages.kUndefinedException;
      }

      if (e.type == DioErrorType.RESPONSE) {
        throw M4eServerException.fromJson(e.response.data);
      }

      throw M4eServerException.fromString(e.message.toString());
    } on SocketException {
      throw M4eExceptionMessages.kNoInternetConnectionException;
    } on HttpException {
      throw M4eExceptionMessages.kHttpException;
    } on FormatException {
      throw M4eExceptionMessages.kHttpFormatException;
    } catch (e) {
      throw M4eServerException.fromString(e.toString());
    }
  }
}
