import 'package:data_connection_checker/data_connection_checker.dart';

import '../../m4e_sdk_initialize.dart';
import '../api/apis.dart';
import '../../m4e_forms/m4e_forms.dart';
import '../../core/core.dart';

/// Exposes SDK Permission Api methods
///
/// Accessing the instantiated `M4ePermissionService` methods
/// is done by:
///
/// ```
/// final _permissionService = M4ePermissionService();
///
/// await _permissionService.setPermission(M4ePermissionForm permissionForm);
/// ```
///
class M4ePermissionService {
  final PermissionApi _permissionApi;
  final DataConnectionChecker _connectionChecker;

  M4ePermissionService({
    PermissionApi permissionApi,
    DataConnectionChecker connectionChecker,
  })  : _permissionApi = permissionApi ?? m4eSDK<PermissionApi>(),
        _connectionChecker =
            connectionChecker ?? m4eSDK<DataConnectionChecker>();

  /// Sets permissions of a subject on an object
  ///
  /// ```
  /// Example: `Entity-wallet-permissions`
  ///
  /// final _encoding = <M4eAccessPermissionSet>[
  ///              EntityWalletAccess.see,
  ///              EntityWalletAccess.pay,
  ///              EntityWalletAccess.view,
  ///              EntityWalletAccess.grant,
  ///              EntityWalletAccess.alter,
  ///              EntityWalletAccess.create,
  ///              EntityWalletAccess.destroy,
  ///           ];
  ///
  /// final _permissionForm = M4ePermissionForm(
  ///  subId: entityId,
  ///  objId: _walletId,
  ///  type: M4eAccessPermissionType.ENTITY_WALLET, // (options: ENTITY_ENTITY, ENTITY_WALLET and WALLET_WALLET),
  ///  encoding: _encoding,
  /// );
  /// ```
  ///
  /// Throws [M4eServerException] if call to server fails
  ///
  /// Throws [M4eServerException] with `NO_INTERNET_CONNECTION` error
  /// code device is not connected to the internet
  ///
  Future<M4eUnitImpl> setPermission(M4ePermissionForm permissionForm) async {
    if ((await _connectionChecker.hasConnection) ?? false) {
      try {
        final _ = await _permissionApi.setPermission(permissionForm);

        return _;
      } catch (e) {
        rethrow;
      }
    }

    throw M4eExceptionMessages.kNoInternetConnectionException;
  }
}
