import 'package:data_connection_checker/data_connection_checker.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:m4e_sdk/src/core/core.dart';
import 'package:m4e_sdk/src/data/api/apis.dart';
import 'package:m4e_sdk/src/data/services/services.dart';
import 'package:m4e_sdk/src/m4e_forms/m4e_forms.dart';
import 'package:m4e_sdk/src/models/models.dart';
import 'package:matcher/matcher.dart';
import 'package:mockito/mockito.dart';

class MockGroupApi extends Mock implements GroupApi {}

class MockDataConnectionChecker extends Mock implements DataConnectionChecker {}

void main() {
  M4eGroupService _m4eGroupService;

  MockGroupApi _mockGroupApi;
  MockDataConnectionChecker _mockDataConnectionChecker;

  setUp(() {
    _mockGroupApi = MockGroupApi();
    _mockDataConnectionChecker = MockDataConnectionChecker();

    _m4eGroupService = M4eGroupService(
      groupApi: _mockGroupApi,
      connectionChecker: _mockDataConnectionChecker,
    );
  });

  final _m4eGroupForm = M4eGroupForm(name: 'Test');

  final _m4eGroupEntityRoleForm = M4eGroupEntityRoleForm(
    groupId:
        M4eUniqueId.fromUniqueString('46e87882-4ff8-4770-9846-870fbbcc35e0'),
    groupEntityRoles: <GroupEntityRole>[
      GroupEntityRole(
        entityId: M4eUniqueId.fromUniqueString(
            '46e87882-4ff8-4770-9846-870fbbcc35e0'),
        role: M4eRole.OWNER,
      )
    ],
  );

  group('m4e-group-test createGroup', () {
    test('should verify device connectivity status', () {
      // arrange
      when(_mockDataConnectionChecker.hasConnection)
          .thenAnswer((_) async => true);

      // act
      _m4eGroupService.createGroup(_m4eGroupForm);

      // assert
      verify(_mockDataConnectionChecker.hasConnection);
    });

    group('isConnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => true);
      });

      test('should return [M4eUniqueId] when call is successfull', () async {
        // arrange
        when(_mockGroupApi.createGroup(any))
            .thenAnswer((_) async => _m4eGroupForm.id);

        // act
        final _response = await _m4eGroupService.createGroup(_m4eGroupForm);

        // assert
        verify(_mockGroupApi.createGroup(_m4eGroupForm));
        expect(_m4eGroupForm.id, _response);
      });

      test('should throw [ServerException] when call is unsuccessfull',
          () async {
        // arrange
        when(_mockGroupApi.createGroup(any))
            .thenThrow(M4eExceptionMessages.kUndefinedException);

        // act
        final _call = _m4eGroupService.createGroup;

        // assert
        expect(() => _call(_m4eGroupForm),
            throwsA(TypeMatcher<M4eServerException>()));
      });
    });

    group('isDisconnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => false);
      });

      test(
          'should throw a ServerException when no internet connection is detected',
          () async {
        // arrange
        when(_mockGroupApi.createGroup(any))
            .thenThrow(M4eExceptionMessages.kNoInternetConnectionException);

        // act
        final _call = _m4eGroupService.createGroup;

        // assert
        expect(() => _call(_m4eGroupForm),
            throwsA(TypeMatcher<M4eServerException>()));
      });
    });
  });

  group('m4e-group-test addGroupEntityRoles', () {
    test('should verify device connectivity status', () {
      // arrange
      when(_mockDataConnectionChecker.hasConnection)
          .thenAnswer((_) async => true);

      // act
      _m4eGroupService.addGroupEntityRoles(_m4eGroupEntityRoleForm);

      // assert
      verify(_mockDataConnectionChecker.hasConnection);
    });

    group('isConnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => true);
      });

      test('should return [List<M4eUniqueId>] when call is successfull',
          () async {
        // arrange
        when(_mockGroupApi.addGroupEntityRoles(any)).thenAnswer(
          (_) async => _m4eGroupEntityRoleForm.groupEntityRoles
              .map((e) => e.entityId)
              .toList(),
        );

        // act
        final _response =
            await _m4eGroupService.addGroupEntityRoles(_m4eGroupEntityRoleForm);

        // assert
        verify(_mockGroupApi.addGroupEntityRoles(_m4eGroupEntityRoleForm));
        expect(
          _m4eGroupEntityRoleForm.groupEntityRoles
              .map((e) => e.entityId)
              .toList(),
          _response,
        );
      });

      test('should throw [ServerException] when call is unsuccessfull',
          () async {
        // arrange
        when(_mockGroupApi.addGroupEntityRoles(any))
            .thenThrow(M4eExceptionMessages.kUndefinedException);

        // act
        final _call = _m4eGroupService.addGroupEntityRoles;

        // assert
        expect(() => _call(_m4eGroupEntityRoleForm),
            throwsA(TypeMatcher<M4eServerException>()));
      });
    });

    group('isDisconnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => false);
      });

      test(
          'should throw a ServerException when no internet connection is detected',
          () async {
        // arrange
        when(_mockGroupApi.addGroupEntityRoles(any))
            .thenThrow(M4eExceptionMessages.kNoInternetConnectionException);

        // act
        final _call = _m4eGroupService.addGroupEntityRoles;

        // assert
        expect(() => _call(_m4eGroupEntityRoleForm),
            throwsA(TypeMatcher<M4eServerException>()));
      });
    });
  });
}
